package ap.proof.goal;

import ap.terfor.{Term, Formula, ConstantTerm, TermOrder}
import ap.terfor.conjunctions.{Conjunction, ConjunctEliminator}
import ap.terfor.substitutions.Substitution
import ap.util.{Debug, FilterIt}

/**
 * Task for removing facts that are no longer needed (like equations that have
 * been applied to all other formulas), or that can be discharged directly by
 * moving them into the contraint.
 */
case object EliminateFactsTask extends EagerTask {

  val AC = Debug.AC_ELIM_FACTS_TASK

  def apply(goal : Goal, ptf : ProofTreeFactory) : ProofTree = {
    val eliminator = new Eliminator(goal.facts, goal, ptf)
    val newFacts = eliminator.eliminate
    
    ////////////////////////////////////////////////////////////////////////////
    
    if (newFacts == goal.facts)
      // nothing has changed and the application of this task was unnecessary
      ptf.updateGoal(goal)
    else
      eliminator.postProcessor(ptf.updateGoal(newFacts, goal))
  }

}


private class Eliminator(oriFacts : Conjunction,
                         goal : Goal, ptf : ProofTreeFactory)
              extends ConjunctEliminator(oriFacts,
                                         for (c <- goal.eliminatedConstants)
                                           yield c.asInstanceOf[Term],
                                         goal.order) {
  
  var postProcessor : ProofTree => ProofTree = ((p) => p)

  protected def nonUniversalElimination(f : Conjunction) =
    postProcessor = postProcessor compose
      ((pt:ProofTree) => ptf.weaken(pt, f, goal.order))
  
  protected def universalElimination(eliminatedConstant : ConstantTerm,
                                     witness : (Substitution, TermOrder) => Substitution) =
    postProcessor = postProcessor compose
      ((pt:ProofTree) => ptf.eliminatedConstant(pt, eliminatedConstant,
                                                witness, goal.order))

  protected def eliminationCandidates(facts : Conjunction) : Iterator[Term] =
    FilterIt(facts.constants.elements,
             (c:ConstantTerm) => !(goal.tasks.taskInfos.constants contains c) &&
                                 !(goal.clauses.constants contains c))

}