package ap.proof.goal;

import ap.terfor.conjunctions.{Conjunction, NegatedConjunctions,
                               Quantifier, ClauseMatcher}
import ap.terfor.preds.PredConj
import ap.util.{Debug, Logic, Seqs}

object NegLitClauseTask {
  
  private val AC = Debug.AC_COMPLEX_FORMULAS_TASK

  /**
   * Return <code>true</code> if <code>f</code> is a formula that can be handled
   * by this task
   */
  def isCoveredFormula(f : Conjunction) : Boolean =
    !f.quans.isEmpty &&
    Logic.forall(for (q <- f.quans.elements) yield (q == Quantifier.EX)) &&
    // TODO: generalise this
    !f.predConj.positiveLits.isEmpty //&&
    // TODO: find a proper condition when nested quantifiers can be allowed here
   // f.negatedConjs.isEmpty

  private def allowedNegConj(conj : Conjunction) : Boolean =
    true
   
}

class NegLitClauseTask(_formula : Conjunction, _age : Int)
                       extends FormulaTask(_formula, _age) {

  val priority : Int = -10000 + age

  //////////////////////////////////////////////////////////////////////////////
  Debug.assertCtor(NegLitClauseTask.AC, !formula.isTrue)
  //////////////////////////////////////////////////////////////////////////////
  
  /**
   * Perform the actual task (whatever needs to be done with <code>formula</code>)
   */
  protected def doTask(goal : Goal, ptf : ProofTreeFactory) : ProofTree = {
    val order = goal.order
    
    val newClauses = NegatedConjunctions(goal.clauses.elements ++
                                         Iterator.single(formula),
                                         order)
    val newTasks =
      if (newClauses == goal.clauses) {
        // then the new clause was subsumed by some other clause
        List[FormulaTask]()
      } else {
        val matcher = new ClauseMatcher(PredConj.TRUE, goal.facts.predConj,
                                        goal.reduceWithFacts, order)
        matcher matchClause formula
        
        Seqs.toRandomAccess(
          for (matchedClause <- matcher.resolvedClauses.elements;
               task <- goal.formulaTasks(matchedClause).elements) yield task)
      }
           
    ptf.updateGoal(newClauses, newTasks, goal)
  }

  /**
   * Create a new <code>FormulaTask</code> by updating the value of
   * <code>formula</code>
   */
  protected def updateFormula(f : Conjunction, goal : Goal) : FormulaTask =
    new NegLitClauseTask(f, age)

  /**
   * Return <code>true</code> if <code>f</code> is a formula that can be handled
   * by this task
   */
  def isCoveredFormula(f : Conjunction) : Boolean =
    NegLitClauseTask isCoveredFormula f

  protected val name : String = "NegLitClause"

}
