package ap.util;

object FilterIt {
  
  def apply[T](baseIt : Iterator[T], pred : T => Boolean) : Iterator[T] =
    new FilterIt(baseIt, pred)
  
}

/**
 * An own class for filtering the elements that are returned by an iterator,
 * because the method <code>Iterator.filter</code> is so broken
 */
class FilterIt[T] private (baseIt : Iterator[T],
                           pred : T => Boolean) extends Iterator[T] {
  private var nextVal : T = _
  private var nextValAvailable : Boolean = false

  private def runToNext = {
    while (!nextValAvailable && baseIt.hasNext) {
      nextVal = baseIt.next
      nextValAvailable = pred(nextVal)
    }
  }

  def hasNext = {
    runToNext
    nextValAvailable
  }

  def next = {
    runToNext
    nextValAvailable = false
    nextVal
  }
}
