package ap.util;

object LazyMappedSet {
  
  private val AC = Debug.AC_SET_UTILS
  
}

/**
 * Transform a set by applying a given injective function to all of its
 * arguments.
 */
class LazyMappedSet[A,B] (oriSet : scala.collection.Set[A],
                          mapping : (A) => B,
                          unmapping : PartialFunction[B,A])
      extends scala.collection.Set[B] {

  def size : Int = oriSet.size
  
  def contains(x : B) : Boolean =
    if (unmapping isDefinedAt x) {
      val oriX = unmapping(x)
      //////////////////////////////////////////////////////////////////////////
      Debug.assertInt(LazyMappedSet.AC, mapping(oriX) == x)
      //////////////////////////////////////////////////////////////////////////
      oriSet contains oriX
    } else {
      false
    }
  
  def elements : Iterator[B] = for (x <- oriSet.elements) yield {
    val mappedX = mapping(x)
    ////////////////////////////////////////////////////////////////////////////
    Debug.assertInt(LazyMappedSet.AC, (unmapping isDefinedAt mappedX) &&
                                      unmapping(mappedX) == x)
    ////////////////////////////////////////////////////////////////////////////  
    mappedX
  }
  
}
