package ap.util;

object UnionSet {
  
  private val AC = Debug.AC_SET_UTILS

  def apply[A](set1 : scala.collection.Set[A],
               set2 : scala.collection.Set[A]) : scala.collection.Set[A] =
    if (set1.size == 0)
      set2
    else if (set2.size == 0)
      set1
    else
      new UnionSet (set1, set2)
}

/**
 * A (lazy) <code>Set</code> that represents the union of two
 * (not necessarily disjoint) <code>Set</code>s
 */
class UnionSet[A] private (set1 : scala.collection.Set[A],
                           set2 : scala.collection.Set[A])
      extends scala.collection.Set[A] {

  def contains(x : A) : Boolean = (set1 contains x) || (set2 contains x)
  
  lazy val size : Int = {
    var res = set1.size
    for (x <- set2) {
      if (!(set1 contains x))
        res = res + 1
    }
    res
  }
  
  def elements : Iterator[A] =
    set1.elements ++ FilterIt(set2.elements, (x:A) => !(set1 contains x))
  
}
